#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_cybak-lzma
----------------------------------

Tests for `cybak-lzma` module.
"""

import unittest

from cybak-lzma import cybak-lzma


class TestCybak-lzma(unittest.TestCase):

    def setUp(self):
        pass

    def test_something(self):
        pass

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



from backup import dirwalker
import unittest
import os
import re

class DirwalkerFixt(unittest.TestCase):

    def setUp(self):
        self.source = "/home/cybernetics/"
        self.file_exceptions = ['~/saserrors.log','~/v.txt']
        self.dir_exceptions = ['~/olduser']

class DirwalkerTest(DirwalkerFixt):

    # test that the first excepted file does not appear
    def test_listing(self):
        lis = dirwalker.file_lister(self.source,
                file_exceptions=self.file_exceptions,
                dir_exceptions=self.dir_exceptions) 
        patt = re.compile(os.path.basename(self.file_exceptions[0]))
        for item in lis:
            print item
            self.assertEqual(patt.search(item), None,
                "Item: " + item)

    def test_dir_exceptions(self):
        lis = dirwalker.file_lister(self.source,
                file_exceptions=self.file_exceptions,
                dir_exceptions=self.dir_exceptions)
        patt = re.compile(os.path.basename(self.dir_exceptions[0]))
        for item in lis:
            self.assertEqual(patt.search(item), None,
                "Dir: " + item)

suite = unittest.TestLoader().loadTestsFromTestCase(DirwalkerTest)
unittest.TextTestRunner(verbosity=2).run(suite)



import unittest
import tempfile

class ConfigFixture(unittest.TestCase):

    def setUp(self):
        # make a config file object with testable properties
        fil = tempfile.TemporaryFile()
        fil.write('[SECTION1]\ndir1=../\ndir2=~\ndirlist=./zsh_src/,/python')
        
class ConfigTests(ConfigFixture):

    def test_configbhv(self):
       pass  



import os
from os.path import join, getsize
from os import path


# returns a list of files which are not in file_exceptions,
# nor is any part of their path in # dir_exceptions;
# hidden files and folders are ignored by default
def file_lister(source, ignore_hidden=True, file_exceptions=[],
                    dir_exceptions=[]):

    # save the current directory and then move to the source directory
    orig_dir = os.getcwd()
    os.chdir(source)

    # build the list of directories to exclude
    exception_list = []
    for exception in dir_exceptions:
        exception_list.append(user_parse(exception))

    # build the list of files to exclude
    ignore_files = []
    for item in file_exceptions:
        ignore_files.append(user_parse(item))

    # begin building the file list
    file_list = []
    for root, dirs, files in os.walk(source,topdown=True):
        dirs[:] = [d for d in dirs if join(root,d) not in exception_list]
        if ignore_hidden == True:
            dirs[:] = [d for d in dirs if not d.startswith('.')]
        for f in files:
            if f.startswith('.'):
                pass
            elif os.path.join(root, f) in ignore_files:
                pass
            else:
                file_list.append(join(root, f))

    # change back to the directory that this process was in at init
    os.chdir(orig_dir)

    return file_list


# a quick function to handle user directory shorthands in a string
def user_parse(file_path):
    item_path = None
    item_base = None
    item_path = os.path.expanduser(os.path.dirname(file_path))
    item_base = os.path.basename(file_path)
    return os.path.join(item_path, item_base)



import sys
import os
from os import path
import re
import tarfile
import logging
import time
import random

from dirwalker import file_lister
import config_handler
import pash
import colorprint
import menu_generator


file_name = os.path.abspath(sys.argv[0])

config_path = os.path.join(os.path.dirname(file_name),'CONFIG.txt')

config = config_handler.Parser(filename=config_path)

# first, check for configuration file
error_msg = ("Didn't find config file in the location specified"
           " in the body of the script!\n"+ "\tPath:\t" + config_path +
            "\nIt should be in the current working directory, with" +
            " the name \'CONFIG.txt\'.")

if config.config_exists(config_path) != True:
    print error_msg
    print ("Please create this file and re-run the script.") 
    print ("An example configuration file has been included" +
            " at the bottom of this script.")
    sys.exit(1)


# exception list
dir_exceptions = config.get_setting('EXCEPT',
                        'DIR_EXCEPT').split(',')
if dir_exceptions[-1] == ['']:
    dir_exceptions.pop()
file_exceptions = config.get_setting('EXCEPT',
                        'FILE_EXCEPT').split(',')
if file_exceptions[-1] == ['']:
    file_exceptions.pop()


# set destination directory
dest_dir = config.get_setting('DIR','DEST_DIR')

source_dir = os.environ['HOME']
# set source root if the setting is non-blank
if config.get_setting('DIR','SRC_DIR') != '':
    source_dir = config.get_setting('DIR','SRC_DIR')

# backup file name
timestamp = time.strftime('%Y%m%d.%H%M')
file_name = (config.get_setting('NAME','FILENAME') + "-" + timestamp
         + '.' + str(random.randint(100,999)))

# make backup directory if it does not already exist
if not os.path.exists(dest_dir):
	print "Creating \'", dest_dir,"\' as a destination for backups."
	os.makedirs(dest_dir)
	assert os.path.exists(dest_dir), "We tried to make a directory, but couldn't find it afterwards."
else:
	print "Found destination directory:", dest_dir

# begin printing exception-related messages
print ""

# set ignore_hidden
ignore_hidden = False
ignore_hidden_str = config.get_setting('EXCEPT','IGNORE_HIDDEN')
if ignore_hidden_str == "True" or ignore_hidden_str == "true":
    print "Ignoring hidden files and folders..."
    ignore_hidden = True

print "Files that will be ignored:"
for exc in file_exceptions:
    print " "*4, exc
print "Directories that will be ignored:"
for exc in dir_exceptions:
    print " "*4, exc
print ""

# build list of files
total_list = file_lister(source_dir, ignore_hidden=True, file_exceptions=file_exceptions, dir_exceptions=dir_exceptions)

# 
print("Building file list and packing tarball...")

# create a tarfile object that is opened for appending,
# then recursively append each target to it
tar = tarfile.open(name=dest_dir+file_name+'.tar',mode='w')
for i in total_list:
	tar.add(name=i,recursive=True)
tar.close()
handle = dest_dir +  file_name + ".tar"

# check filesize for consent from user
proc = pash.ShellProc()
proc.run("du -sh " + handle + " | column -t|cut -d ' ' -f 1")
size = proc.get_val('stdout').rstrip('\n')
print("The total size of the files in the built tarball is"),
colorprint.color_out(size, int(96), '.\n')
print("The compressed backup will likely be smaller than this.\n")

if len(sys.argv) > 1:
    if sys.argv[1] == 'auto':
        result = True
else:
    yn = menu_generator.YN_Menu(default="no")
    result = yn.run()
if result == False:
    print("Operation cancelled, deleting tarball.")
    os.remove(handle)
    sys.exit(1)

print("\nApplying lzma compression... (this may take several minutes)")

command = 'lrzip ' + handle
proc.run(command)
data = proc.get_val('stdout').rstrip('\n').split('\n')

print("Compression complete.")

# waits for the backup file to exist, then removes the tar
print ("Ensuring the backup file exists..."),
while os.path.exists(handle+".lrz") == False:
	pass
print("found it!\n")

print data[0]

command = "du -sh " + data[0].split(':')[1].lstrip(' ') + "| column -t | cut -d ' ' -f 1"
proc.run(command)
print "Compressed backup size: ", 
colorprint.color_out(proc.get_val('stdout'), int(96))


for index, i in enumerate(data[1].split(' ')):
	if i == 'Compression' and data[1].split(' ')[index+1] == 'Ratio:':
		comp_ratio = data[1].split(' ')[index+2].rstrip('.')

print "Compression ratio:", 
colorprint.color_out(comp_ratio, int(96))


print("\nDeleting the uncompressed tarball.")
os.remove(handle)

print("Backup complete. Have a nice day.")


# begin sample configuration file
# remove the first layer of hashes to use

#[DIR]
## if empty, SRC_DIR will default to the "home" directory of the
## current user
#SRC_DIR=
#DEST_DIR=/backup/my-backup/
#
#
#[EXCEPT]
## exceptions should be comma-separated
## example: 
##   EXCEPTIONS=/this/dir,/noman/dir,
##
#DIR_EXCEPT=/zfs_src,
#FILE_EXCEPT=
#
## IGNORE_HIDDEN will cause the script to ignore any file or directory
## that begins with a period
#IGNORE_HIDDEN=True
#
#
#[NAME]
## the output filename for the backup will begin with the following string
## example:
##   FILENAME=MyBackup
##
##   results in a file: MyBackup-20150106.1845.tar.lrz
#FILENAME=MyBackup
